<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Cases extends MY_Controller
{
	public function __construct()
	{
		parent::__construct();

		$this->load->library('c_case');
		$this->load->library('form_validation');
		$this->form_validation->set_rules('mode', 'Mode', 'required');
	}

	public function init()
	{
		$this->load->config('templates', TRUE);
		
		$case = $this->c_case->load();

		if (empty($case))
		{
			$case = array();
			$attr = array();
			$template = key($this->config->item('templates'));
		}
		else
		{
			$attr = $case['attr'];
			$template = $attr['template'];
		}

		if ($this->form_validation->run() === TRUE)
		{
			$this->load->config('case', TRUE);

			$attr = array('template' => $this->input->post('template'));

			$template = $this->config->item('templates')[$attr['template']];
			$info = array('Template' => $template['title']);

			foreach ($this->config->item('case')['attr'] as $key => $val)
			{
				if (empty($template[$key]))
				{
					$attr[$key] = $this->input->post($key);
				}
				else
				{
					$attr[$key] = $template[$key];
				}

				$info[$val['title']] = $attr[$key];
			}
			
			$case['attr'] = $attr;
			$case['info'] = $info;
			$this->c_case->init_case($case);

			$this->c_workspace->redirect('cases/view');
		}

		$js = "$(document).ready(function() { $(template).change(function(e) { switch ($(this).val()) {";
		foreach ($this->config->item('templates') as $temp => $info)
		{
			$opt = '';
			foreach ($info['attr']['tur_model'] as $tur)
			{
				$opt .= '<option value="'.$tur.'">'.$tur.'</option>';
			}
			$js .= "case '".$temp."': $('#tur_model').html('".$opt."'); break;";
        }
		$js .= "} }); });";

		$this->set_header('Setup Case', Array(), $js);
		$this->set_data('attr', $attr);
		$this->set_data('templates', $this->config->item('templates'));
		$this->set_data('template', $template);
		$this->load_case_form('case/init', 'Apply');
	}

	public function view()
	{
		$case = $this->c_case->load();

		$procs = 0;
		if ($this->form_validation->run() === TRUE)
		{
			if ($this->input->post('mode') == 'Solve')
			{
				$case['run_options'] = array('potential' => $this->input->post('potential'), 'parallel' => $this->input->post('parallel'));
				$this->c_case->save_case($case);

				$solve = '';
				if ($this->input->post('potential') == 'on')
				{
					$solve = "potentialFoam\n";
				}
				$solver = $this->config->item('solvers')[$case['attr']['solver']]['solver'];
				$this->file_system->write_data_file('solve', $solve.$solver."\n");

				if ($this->input->post('parallel') == 'on')
				{
					$this->c_workspace->redirect('solve/decompose');
				}
				else
				{
					$this->c_workspace->redirect('solve/run');
				}
			}
		}

		if ($this->job())
		{
			return;
		}

		if (empty($case))
		{
			$this->c_workspace->redirect('cases/init');
		}

		if (empty($case['bnd_setup']))
		{
			$this->c_workspace->redirect('mesh/view');
		}

		$this->c_case->update_inputs();

		$this->set_header('Setup Conditions');
		$this->set_data('options', element('run_options', $case));
		$this->load_case_form('case/view', 'Solve');
	}

	public function delete()
	{
		$this->c_case->load();
		$this->c_case->delete();
		$this->c_workspace->reload(TRUE);
	}

	public function download($name)
	{
		$this->c_case->download($name);
	}

	public function copy()
	{
		$this->c_case->load();

		if ($this->form_validation->run() === TRUE)
		{
			$this->c_case->copy_to($this->input->post('path').DIRECTORY_SEPARATOR.$this->input->post('name'));
			$this->c_workspace->reload(TRUE);
		}

		$server = $this->c_workspace->get_server();
		$path = $this->c_workspace->get_parent_path();
		$js = "$(document).ready(function() { load_file_list('".$server."', $('#dir_path').val(), ''); $('#dir_select').change(function() { load_file_list('".$server."', $('#dir_path').val(), $(this).val()); }); ";
		$js .= "$('form').submit(function(event) { var failed = false; if ($('#name').val() == '') { alert('Name field is required.'); failed = true; } else { $.each(files, function(type, list) { $.each(list, function(index, file) { if ($('#name').val() == file) { alert('Case name is alreay used'); failed = true; } }); }); }  if (failed) { event.preventDefault(); } }); });";
		$js .= "function set_file_list(path, list) { $('#dir_path').val(path); $('#path').val('".$this->file_system->get_wroot()."' + (path == '' ? '' : '/' + path)); set_dir_select(path, list['dirs']); }";

		$this->set_header('Copy Case', array('/js/file_list.js'), $js);
		$this->set_data('path', $path);
		$this->load_case_form('case/copy', 'Copy');
	}

	public function job()
	{
		$this->load->library('c_job');

		$job = $this->c_job->update();

		if ($job)
		{
			switch ($job['stat'])
			{
				case JOB_RUNNING:
					$this->c_workspace->redirect($this->c_job->get_monitor_url());
					break;
				case JOB_DONE:
					$this->c_workspace->redirect($this->c_job->get_result_url());
					break;
				case JOB_ERROR:
					$this->c_workspace->redirect($this->c_job->get_error_url());
					break;
			}
		}
	}
}
