<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Condition extends MY_Controller
{
	public function __construct()
	{
		parent::__construct();

		$this->load->library('c_workspace');
		$this->load->library('c_case');
		$this->load->library('foam_input');
		$this->load->library('form_validation');
		$this->form_validation->set_rules('mode', 'Mode', 'required');
	}

	public function general($sub)
	{
		$case = $this->c_case->load();

		$this->load->config('gen_inputs', TRUE);
		
		$config = $this->config->item('gen_inputs')[$sub];
		$this->load->library('general_input', $config);

		$this->setup('general', $sub, $config['title'], $this->general_input);
	}

	public function boundary($sub)
	{
		$case = $this->c_case->load();

		$this->load->config('bnd_inputs', TRUE);
		$config = $this->config->item('bnd_inputs')[$sub];

		$params['config'] = $config;
		$params['boundaries'] = $case['boundaries'];
		$params['inputs_info'] = $case['inputs']['boundary'][$sub];

		$this->load->library('variable_input');
		switch ($config['class'])
		{
			case 'volVectorField':
				$this->load->library('vector_input', $params);
				$this->setup('boundary', $sub, $config['title'], $this->vector_input);
				break;
			case 'volScalarField':
				$this->load->library('scalar_input', $params);
				$this->setup('boundary', $sub, $config['title'], $this->scalar_input);
				break;
		}
	}

	public function numerical($sub)
	{
		$case = $this->c_case->load();

		$this->load->config('num_inputs', TRUE);
		$config = $this->config->item('num_inputs');

		switch ($sub)
		{
			case 'fvSchemes':
				$this->load->library('schemes_input');
				$this->setup('numerical', $sub, $config[$sub]['title'], $this->schemes_input);
				break;
			case 'fvSolution':
				$this->load->library('solution_input', array('boundary_inputs' => $case['inputs']['boundary']));
				$this->setup('numerical', $sub, $config[$sub]['title'], $this->solution_input);
				break;
		}
	}

	public function run($sub)
	{
		$case = $this->c_case->load();

		$this->load->config('run_inputs', TRUE);
		$config = $this->config->item('run_inputs');

		switch ($sub)
		{
			case 'controlDict':
				$this->load->library('control_input');
				$this->setup('run', $sub, $config[$sub]['title'], $this->control_input, 'control');
				break;
			case 'dynamicMeshDict':
				$this->load->library('dynamic_mesh_input');
				$this->setup('run', $sub, $config[$sub]['title'], $this->dynamic_mesh_input, 'condition', TRUE);
				break;
		}
	}

	private function setup($category, $sub, $title, $input, $view = 'condition', $multipart = FALSE)
	{
		if ($this->form_validation->run() === TRUE)
		{
			$input->submit_form();
			$this->c_case->update_input($category, $sub);
			$this->c_workspace->redirect($this->input->post('next'));
		}
			
		$input_items = $input->load();

		$alert = NULL;
		if (!empty($input_items['error']))
		{
			$alert = array('type' => 'danger', 'title'=> 'NEED TO SAVE!');
			if ($input_items['error']['code'] == 1)
			{
				$alert['msg'] = 'These data have not been saved.';
			}
			else
			{
				$alert['msg'] = 'The input file has wrong data.<br/>['.$input_items['error']['code'].'] '.$input_items['error']['msg'];
			}
		}

		$this->set_header($title, array('/js/inputs.js'), $input->get_js());
		$this->set_data('alert', $alert);
		$this->set_data('items', $input_items['inputs']);
		$this->load_case_form('case/'.$view, 'Save', array(), $multipart);
	}
}
