<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Mesh extends MY_Controller
{
	public function __construct()
	{
		parent::__construct();

		$this->load->library('c_workspace');
		$this->load->library('c_case');
		$this->load->library('form_validation');
		$this->form_validation->set_rules('mode', 'Mode', 'required');
	}

	public function view()
	{
		$case = $this->c_case->load();

		if ($this->form_validation->run() === TRUE)
		{
			switch ($this->input->post('type'))
			{
				case 'confirm_setup':
					$this->c_workspace->redirect('mesh/condition');
					break;
				case 'edit':
					$this->c_workspace->redirect('mesh/edit');
				case 'transformPoints':
					$this->c_workspace->redirect('mesh/transform_points');
				case 'merge':
					$this->file_system->exec_foam("mergeMeshes -overwrite . ".$this->file_system->get_wpath('..').DIRECTORY_SEPARATOR.$this->input->post('case_select'));
					$this->c_workspace->redirect('mesh/import');
				case 'copy':
					$this->c_case->copy_boundary($this->input->post('case_select'));
					$this->c_workspace->redirect('mesh/import');
			}
		}

		if (empty($case['boundaries']))
		{
			$this->c_workspace->redirect('mesh/import');
		}

		$files = $this->file_system->get_files('..');

		$this->set_header('Setup Boundary');
		$this->set_data('boundaries', $case['boundaries']);
		$this->set_data('cases', $files['cases']);
		$this->load_case_form('mesh/view', 'Process', array(array('name' => 'upload', 'value' => 'Upload', 'action' => "if (confirm('Would you delete all polyMesh files?')) location.href='".$this->c_workspace->make_url('/mesh/clear')."'")));
	}

	public function upload()
	{
		$this->c_case->load();

		if ($this->form_validation->run() === TRUE)
		{
			$type = $this->input->post('type');

			if ($type == 'foamMesh')
			{
				$mesh = array('type' => $type);
				$this->c_case->save_mesh($mesh);
				$this->c_workspace->redirect('mesh/import');
			}
			else
			{
				$mesh = array(
					'type'		=> $type,
					'file'		=> $this->input->post('file'),
					'convert'	=> $this->config->item('mesh_converters')[$type]
				);

				$this->c_case->save_mesh($mesh);

				$this->file_system->exec_foam($mesh['convert'].' '.$mesh['file']);
				$this->c_workspace->redirect('cases/view');
			}
		}
		else
		{
			$this->set_alert(form_error(key(_get_validation_object()->error_array()), null, null));
		}

		$this->set_header('Upload Mesh', array('/js/plupload.full.min.js'));
		$this->set_data('converters', $this->config->item('mesh_converters'));
		$this->load_case_form('mesh/upload', 'Upload');
	}

	public function edit()
	{
		$case = $this->c_case->load();

		$this->load->config('bnd_inputs', TRUE);

		$this->load->library('foam_input');
		$this->load->library('boundary_input');

		$input_items = $this->boundary_input->load();

		if (!empty($input_items['error']))
		{
			$alert['type'] = 'danger';
			$alert['title'] = 'PARSE ERROR!';
			$alert['msg'] = 'Fail to parse mesh file : '.$input_items['error']['code'];

			$this->set_header('Edit Boundary');
			$this->set_data('alert', $alert);
			$this->load_case_form('case/process', 'Upload');

			return;
		}

		if ($this->form_validation->run() === TRUE)
		{
			if ($this->input->post('mode') == 'Upload')
			{
				$this->c_workspace->redirect('mesh/upload');
			}

			$this->boundary_input->submit_form();
			$this->c_workspace->redirect('mesh/view');
		}

		$js = "$(document).ready(function() { $(\"select[id$='_type']\").change(function() { var target = $(this)[0].id.slice(0, -5);";
		$js .= "if ($(this).val() == 'cyclicAMI') { $('#' + target + '_cyclicAMI').show(); } else { $('#' + target + '_neighbourPatch').val(''); $('#' + target + '_cyclicAMI').hide();";
		$js .= "$(\"select[id$='_neighbourPatch\").each(function() { if ($(this).val() == target) $(this).val(''); }); } });";
		$js .= "$(\"select[id$='_neighbourPatch']\").change(function() { var target = '#' + $(this).val(); $(target + '_type').val('cyclicAMI'); $(target + '_type').trigger('change'); });";
		$js .= "$('form').submit(function(event) { var msg = ''; $(\"input[id$='_name\").each(function() { var target = $(this)[0].id.slice(0, -5);";
		$js .= "if ($(this).val() == '' && msg == '') msg = '\"' + target + '.Name\" field is required'; if ($('#' + target + '_type').val() == 'cyclicAMI' && msg == '') {";
		$js .= "if ($('#' + target + '_matchTolerance').val() == '' && msg == '') msg = '\"' + target + '.Match Tolerance\" field is required';";
		$js .= "if ($('#' + target + '_neighbourPatch').val() == '' && msg == '') msg = '\"' + target + '.Neighbour Patch\" field is required'; } target = $(this);";
		$js .= "$(\"input[id$='_name\").each(function() { if (target.val() == $(this).val() && target[0].id != $(this)[0].id && msg == '') msg = 'Boundary name \"' + $(this).val() + '\" is duplocated'; }); });";
		$js .= "if (msg != '') { alert(msg); event.preventDefault(); } }); });";

		$this->set_header('Edit Boundary', array(), $js);
		$this->set_data('alert', NULL);
		$this->set_data('items', $input_items['inputs']);
		$this->load_case_form('mesh/import', 'Save');
	}

	public function import()
	{
		$case = $this->c_case->load();

		if ($this->form_validation->run() === TRUE)
		{
			$this->c_workspace->redirect('mesh/upload');
		}

		$error = $this->c_case->import_mesh();
		if (!empty($error))
		{
			if ($error['code'] == 1)
			{
				$mesh = $this->c_case->load_mesh();
				if ($mesh['type'] == 'foamMesh')
				{
					$alert['type'] = 'danger';
					$alert['title'] = 'PARSE ERROR!';
					$alert['msg'] = 'Boundary file does not found';

					$this->set_header('Import Mesh');
					$this->set_data('alert', $alert);
					$this->set_data('log', 'boundary file was not uploaded');
					$this->load_case_form('case/process', 'Upload');

					return;
				}
				else
				{
					$this->c_workspace->redirect('mesh/upload');
				}
			}

			$alert['type'] = 'danger';
			$alert['title'] = 'PARSE ERROR!';
			$alert['msg'] = 'Fail to parse mesh file : '.$error['code'];

			$this->set_header('Import Mesh');
			$this->set_data('alert', $alert);
			$this->set_data('log', $error['msg']);
			$this->load_case_form('case/process', 'Upload');

			return;
		}

		$this->c_workspace->redirect('mesh/view');
	}

	public function condition()
	{
		$case = $this->c_case->load();

		if (empty($case['boundaries']))
		{
			$this->c_workspace->redirect('mesh/import');
		}

		$var_options = $this->c_case->get_var_options();

		if ($this->form_validation->run() === TRUE)
		{
			foreach($case['boundaries'] as $name => $info)
			{
				foreach ($var_options as $var => $options)
				{
					$case['boundaries'][$name][$options['boundary_cond']] = $this->input->post($name.'_'.$var);
				}
			}
			$this->c_case->setup_mesh($case);

			$this->c_workspace->redirect('cases/view');
		}
		else
		{
			$this->set_alert(form_error(key(_get_validation_object()->error_array()), null, null));
		}

		$this->load->config('boundary_types', TRUE);
		$this->load->config('variable_conditions', TRUE);

		$this->set_header('Setup Boundary Conditions');
		$this->set_data('var_options', $var_options);
		$this->set_data('types', $this->config->item('boundary_types'));
		$this->set_data('boundaries', $case['boundaries']);
		$this->load_case_form('mesh/condition', 'Save');
	}

	public function transform_points()
	{
		$case = $this->c_case->load();

		$this->load->library('util_input');

		if ($this->form_validation->run() === TRUE)
		{
			$this->file_system->exec_foam("transformPoints -rollPitchYaw '".$this->util_input->get_form_vector('degrees')."'");
			$this->c_workspace->redirect('mesh/view');
		}

		$inputs = $this->util_input->transform_points();
		$this->set_header('Transform Points', array('/js/inputs.js'), $inputs['js']);
		$this->set_data('items', $inputs['items']);
		$this->load_case_form('case/util', 'Transform');
	}

	public function clear()
	{
		$case = $this->c_case->load();
		$this->c_case->clear_mesh();
		$this->c_workspace->redirect('mesh/upload');
	}
}
