<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Solve extends MY_Controller
{
	public function __construct()
	{
		parent::__construct();

		$this->load->library('c_workspace');
		$this->load->library('c_case');
		$this->load->library('foam_input');
		$this->load->library('form_validation');
		$this->form_validation->set_rules('mode', 'Mode', 'required');
	}

	public function decompose()
	{
		$case = $this->c_case->load();

		$this->load->library('decompose_par_input');

		if ($this->form_validation->run() === TRUE)
		{
			$this->decompose_par_input->submit_form();
			$this->run($this->input->post('numberOfSubdomains'));
		}

		$input_items = $this->decompose_par_input->load();
		$alert = NULL;

		$this->set_header('Run In Parallel', array('/js/inputs.js'), $this->decompose_par_input->get_js());
		$this->set_data('alert', $alert);
		$this->set_data('items', $input_items['inputs']);
		$this->load_case_form('case/condition', 'Solve');
	}
	
	public function run($procs = 0)
	{
		$this->load->library('c_job');
		$this->c_job->start('solve', 'solve', 'solve', $procs);
	}

	public function monitor()
	{
		$case = $this->c_case->load();

		$this->load->library('c_job');
		$job = $this->c_job->get_job();

		if ($this->form_validation->run() === TRUE)
		{
			$this->load->library('c_job');

			if ($this->input->post('mode') == 'Stop')
			{
				$this->c_job->stop();
			}
			else if ($this->input->post('mode') == 'WriteNow')
			{
				$this->c_case->write_now();
			}

			$this->c_workspace->reload();
		}

		$job = $this->c_job->update();

		switch ($job['stat'])
		{
			case JOB_RUNNING:
				$alert['type'] = 'info';
				$alert['title'] = 'Running!';
				$alert['msg'] = 'Process is running.';

				$this->load_solve_view($alert, 'Refresh', array(array('name' => 'stop', 'value' => 'Stop', 'action' => 'submit'), array('name' => 'writeNow', 'value' => 'WriteNow', 'action' => 'submit')));
				break;
			case JOB_DONE:
				$this->c_workspace->redirect($this->c_job->get_result_url());
				break;
			case JOB_ERROR:
				$this->c_workspace->redirect($this->c_job->get_error_url());
				break;
			default:
				$this->c_workspace->redirect('cases/view');
		}
	}

	public function result()
	{
		$case = $this->c_case->load();

		$this->load->library('c_job');

		if ($this->form_validation->run() === TRUE)
		{
			$this->c_job->clear();
			$this->c_workspace->redirect('cases/view');
		}

		$alert['type'] = 'success';
		$alert['title'] = 'Finished!';
		$alert['msg'] = 'Process is finished.';

		$this->load_solve_view($alert, 'Finish');
	}

	public function fail()
	{
		$case = $this->c_case->load();

		$this->load->library('c_job');
		$job = $this->c_job->get_job();

		if ($this->form_validation->run() === TRUE)
		{
			$this->c_job->clear();
			$this->c_workspace->redirect('/cases/view');
		}

		$alert['type'] = 'danger';
		$alert['title'] = 'Error!';
		$alert['msg'] = 'Process is failed : '.$job['result'];

		$this->set_header('Solve Case');
		$this->set_data('alert', $alert);
		$this->set_data('log', $this->c_job->tail_log());
		$this->load_case_form('case/process', 'Finish');
	}

	private function load_solve_view($alert, $submit, $buttons = array())
	{
		$this->load->library('solve_log');
		$chart_data = $this->solve_log->get_chart_data();
		
		if (empty($chart_data))
		{
			$this->set_header('Solve Case');
			$this->set_data('alert', $alert);
			$this->set_data('log', $this->c_job->tail_log());
			$this->load_case_form('case/process', $submit, $buttons);
		}
		else
		{
			$this->set_header('Solve Case', array('/js/Chart.min.js', '/js/monitor.js'), '$(document).ready(function() { drawChart('.$chart_data.') });');
			$this->set_data('alert', $alert);
			$this->load_case_form('case/solve', $submit, $buttons);
		}
	}
}
