<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Workspace extends MY_Controller
{
	public function __construct()
	{
		parent::__construct();

		if ($this->file_system->is_case())
		{
			$this->c_workspace->redirect('cases/view');
		}
	}

	public function index()
	{
		$this->c_workspace->reload();
	}

	public function view()
	{
		$this->load->helper('form');
		$this->load->library('form_validation');

		$this->form_validation->set_rules(
			'name',
			'Name',
			array(
				'required',
				array('dir_name_start', array($this->c_workspace, 'check_dir_name_start')),
				array('dir_name_char', array($this->c_workspace, 'check_dir_name_char')),
				array('dir_uniq', array($this->c_workspace, 'check_dir_uniq')),
			),
			array(
				'dir_name_start' => 'Directory name cannot start with dot',
				'dir_name_char' => 'Directory name cannot include space or slash',
				'dir_uniq' => 'Directory or case already exists'
			)
		);

		if ($this->form_validation->run() === TRUE)
		{
			switch ($this->input->post('type'))
			{
				case 'directory':
					$this->c_workspace->create_dir($this->input->post('name'));
					$this->c_workspace->reload();
					break;
				case 'case':
					$this->c_workspace->create_case($this->input->post('name'));
					$this->c_workspace->redirect('cases/view', $this->input->post('name'));
					break;
				case 'copy':
					$this->c_workspace->copy_case($this->input->post('name'), $this->input->post('case_sel'));
					$this->c_workspace->reload();
			}
		}
		else
		{
			$this->set_alert(form_error(key(_get_validation_object()->error_array()), null, null));
		}

		$list = $this->c_workspace->list_dir();

		$this->set_header('Workspace');
		$this->set_data('servers', $this->config->item('foam_server'));
		$this->set_data('server', $this->input->get('server'));
		$this->set_data('dirs', $list['dirs']);
		$this->set_data('cases', $list['cases']);
		$this->load_view('case/list');
	}

	public function delete()
	{
		$this->c_workspace->delete();
		$this->c_workspace->reload(TRUE);

	}
}
