<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class C_case
{
	protected $CI;

	protected $case;

	public function __construct()
	{
		$this->CI =& get_instance();

		$this->CI->load->config('solvers', TRUE);
	}

	public function load()
	{
		if(!$this->CI->file_system->is_case())
		{
			show_error("The case you requested was not found.", 404, "Invalid Request");
		}

		$this->case = $this->CI->file_system->load_data('case');

		return $this->case;
	}

	public function init_case($case)
	{
		$this->case = $case;
		$this->CI->file_system->exec_util('init_case', $this->CI->config->item('templates')[$case['attr']['template']]['template']);

		$case['inputs'] = array(
			'general' => $this->get_inputs('gen_inputs'),
			'boundary' => $this->get_boundary_inputs(),
			'numerical' => $this->get_inputs('num_inputs'),
			'run' => $this->get_inputs('run_inputs')
		);

		$this->save_case($case);
	}

	public function get_case()
	{
		return $this->case;
	}

	public function get_attr($key)
	{
		return $this->case['attr'][$key];
	}

	public function set_boundaries($data)
	{
		$boundaries = array();

		foreach ($data as $boundary => $info)
		{
			$boundaries[$boundary] = array('type' => $info['type']);
		}

		$this->case['boundaries'] = $boundaries;

		$this->save_case($this->case);
	}

	public function save_case($case)
	{
		$this->case = $case;
		$this->CI->file_system->save_data('case', $case);
	}

	public function save_mesh($mesh)
	{
		$this->CI->file_system->save_data('mesh', $mesh);
	}

	public function load_mesh()
	{
		return $this->CI->file_system->load_data('mesh');
	}

	public function delete()
	{
		$this->CI->file_system->delete();
	}

	public function check_condition($item)
	{
		if (empty($item['cond']))
		{
			return TRUE;
		}

		foreach($item['cond'] as $conds)
		{
			$res = TRUE;
			foreach($conds as $key => $values)
			{
				if (!in_array($this->case['attr'][$key], $values))
				{
					$res = FALSE;
					break;
				}
			}

			if ($res)
			{
				return TRUE;
			}
		}

		return FALSE;
	}

	public function get_var_options()
	{
		$this->CI->load->config('variable_conditions', TRUE);

		$var_info = $this->CI->config->item('variable_conditions');
		$solver = $this->CI->config->item('solvers')[$this->case['attr']['solver']];

		$vars = $solver['variables'];
		foreach ($solver['variables'] as $var => $info)
		{
			$vars[$var]['var_info'] = $var_info[$info['boundary_cond']];
		}

		$tur_model = $this->case['attr']['tur_model'];
		$vars[$tur_model] = $solver['tur_model'][$tur_model];
		$vars[$tur_model]['var_info'] = $var_info[$vars[$tur_model]['boundary_cond']];

		return $vars;
	}

	public function update_inputs()
	{
		$files = $this->CI->file_system->find_files();

		foreach ($this->case['inputs'] as $category => $inputs)
		{
			foreach ($inputs as $var => $info)
			{
				$this->case['inputs'][$category][$var]['saved'] = in_array($info['file'], $files) ? INP_SAVED : !empty($info['optional']);
				$this->case['inputs'][$category][$var]['active'] = FALSE;
			}
		}

		$this->save_case($this->case);
	}

	public function update_input($category, $input)
	{
		$this->case['inputs'][$category][$input]['saved'] = INP_SAVED;
		$this->save_case($this->case);
	}

	public function import_mesh()
	{
		$this->CI->load->library('foam_db');
		$this->CI->load->library('boundary_db');
	
		$boundaries = $this->CI->boundary_db->get_boundaries();
		if (empty($boundaries))
		{
			return $this->CI->boundary_db->get_error();
		}

		$this->case['boundaries'] = $boundaries;
		$this->save_case($this->case);

		return NULL;
	}

	public function setup_mesh($case)
	{
		$case['bnd_setup'] = TRUE;

		$this->save_case($case);
	}

	public function upload($path, $file)
	{
		return $this->CI->file_system->upload($path, $file);
	}

	public function download($name)
	{
		$this->CI->load->helper('download');

		force_download($this->CI->file_system->compress($name), NULL);
	}

	public function copy_boundary($case)
	{
		$this->CI->file_system->exec_util('copy_boundary', '..'.DIRECTORY_SEPARATOR.$case);
	}

	public function clear_mesh()
	{
		unset($this->case['boundaries']);
		unset($this->case['bnd_setup']);
		$this->save_case($this->case);
		$this->CI->file_system->empty_dir($this->CI->config->item('directory')['polyMesh']);
		$this->CI->file_system->empty_dir('0');
	}

	public function write_now()
	{
		$this->CI->file_system->exec_util('edit_dict', 'system/controlDict stopAt writeNow');
	}

	private function get_inputs($config)
	{
		$inputs = array();

		$this->CI->load->config($config, TRUE);

		foreach ($this->CI->config->item($config) as $name => $condition)
		{
			if ($this->check_condition($condition))
			{
				$inputs[$name] = array('title' => $condition['title'], 'file' => $condition['location'].DIRECTORY_SEPARATOR.$condition['object'], 'optional' => !empty($condition['optional']));
			}
		}

		return $inputs;
	}

	private function get_boundary_inputs()
	{
		$inputs = array();

		$this->CI->load->config('bnd_inputs', TRUE);
		$this->CI->load->config('variable_conditions', TRUE);

		$var_info = $this->CI->config->item('variable_conditions');
		$solver = $this->CI->config->item('solvers')[$this->case['attr']['solver']];
		$variables = $this->CI->config->item('bnd_inputs');

		foreach ($solver['variables'] as $var => $info)
		{
			foreach ($var_info[$info['variables']]['inputs'] as $input)
			{
				$inputs[$input] = array('title' => $variables[$input]['title'], 'file' => $variables[$input]['location'].DIRECTORY_SEPARATOR.$variables[$input]['object'], 'variables' => $var, 'boundary_cond' => $info['boundary_cond']);
			}
		}

		$tur_model = $this->case['attr']['tur_model'];
		$model_info = $solver['tur_model'][$tur_model];
		foreach ($var_info[$model_info['variables']]['inputs'] as $input)
		{
			$inputs[$input] = array('title' => $variables[$input]['title'], 'file' => $variables[$input]['location'].DIRECTORY_SEPARATOR.$variables[$input]['object'], 'variables' => $model_info['variables'], 'boundary_cond' => $model_info['boundary_cond']);
		}
		if (!empty($var_info[$model_info['variables']]['opt_inputs']))
		{
			foreach ($var_info[$model_info['variables']]['opt_inputs'] as $input)
			{
				$inputs[$input] = array('title' => $variables[$input]['title'], 'file' => $variables[$input]['location'].DIRECTORY_SEPARATOR.$variables[$input]['object'], 'variables' => $model_info['variables'], 'boundary_cond' => $model_info['boundary_cond'], 'optional' => TRUE);
			}
		}

		return $inputs;
	}
}
