<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class C_job
{
	protected $CI;

	private $job;
	private $conf;

	public function __construct()
	{
		$this->CI =& get_instance();

		$this->CI->load->config('job', TRUE);

		$this->job = $this->CI->file_system->load_data('job');

		if ($this->job)
		{
			$this->conf = $this->CI->config->item('job')[$this->job['name']];
		}
		else
		{
			$this->job = array('stat' => JOB_NONE);
		}
	}

	public function get_job()
	{
		return $this->job;
	}

	public function get_monitor_url()
	{
		return $this->conf['monitor_url'];
	}

	public function get_result_url()
	{
		return $this->conf['result_url'];
	}

	public function get_error_url()
	{
		return $this->conf['error_url'];
	}

	public function get_title()
	{
		return $this->conf['title'];
	}

	public function start($name, $util, $cmd, $option)
	{
		$this->conf = $this->CI->config->item('job')[$name];

		$this->job = array(
			'stat'		=> JOB_NONE,
			'name'		=> $name,
			'util'		=> $util,
			'cmd'		=> $cmd,
			'option'	=> $option
		);
		
		$this->run();
	}

	public function restart()
	{
		$this->clear();
		$this->run();
	}

	public function update()
	{
		switch (element('stat', $this->job, JOB_NONE))
		{
			case JOB_NONE:
				return NULL;
			case JOB_RUNNING:
				if ($this->is_running())
				{
					break;
				}

				$result = $this->get_result();
				if (trim($result) !== '0')
				{
					$this->job['stat'] = JOB_ERROR;
					$this->job['result'] = $result;
				}
				else
				{
					$this->job['stat'] = JOB_DONE;
					$this->job['result'] = 0;
				}
			case JOB_DONE:
				break;
			default:
				return $this->job;
		}

		return $this->save();
	}

	public function clear()
	{
		$this->stop();

		if (!$this->is_running())
		{
			$this->job['stat'] = JOB_NONE;

			$this->save();
		}
	}

	public function stop()
	{
		$this->CI->file_system->exec_util($this->conf['stop'], $this->job['pid'], TRUE);
	}

	public function tail_log($line_num = 20)
	{
		return $this->CI->file_system->tail($this->job['util'].'.log', $line_num);
	}

	public function get_result()
	{
		return $this->CI->file_system->read_data_file($this->job['util'].'.res');
	}

	private function save()
	{
		$this->CI->file_system->save_data('job', $this->job);

		return $this->job;
	}

	private function run()
	{
		$this->job['pid'] = $this->CI->file_system->exec_util($this->conf['start'], $this->job['option'].' '.$this->job['cmd'])[0];

		$this->job['stat'] = JOB_RUNNING;
		$this->save();

		$this->CI->c_workspace->redirect($this->get_monitor_url());
	}

	private function is_running()
	{
		return $this->CI->file_system->exec_util('check_process', $this->job['pid'], TRUE) !== FALSE;
	}
}
