<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class C_workspace 
{
	protected $CI;

	protected $path;
	protected $name;
	protected $parent_path;

	public function __construct()
	{
		$this->CI =& get_instance();

		$this->CI->load->library('file_system', array('user' => $this->CI->session->user['name']));
		if (!$this->CI->file_system->is_dir())
		{
			show_error("The directory or case you requested was not found.", 404, "Invalid Request");
		}

		$this->path = $this->CI->input->get('path');

		$pos = strrpos($this->path, DIRECTORY_SEPARATOR);
		if ($pos === FALSE)
		{
			$this->name = $this->path;
			$this->parent_path = '';
		}
		else
		{
			$this->name = substr($this->path, $pos + 1);
			$this->parent_path = substr($this->path, 0, $pos);
		}
	}

	public function is_root()
	{
		return $this->CI->file_system->is_root();
	}

	public function get_name()
	{
		return $this->name;
	}

	public function get_path()
	{
		return $this->path;
	}

	public function get_parent_path()
	{
		return $this->parent_path;
	}

	public function list_dir()
	{
		return $this->CI->file_system->get_files();
	}

	public function get_path_links()
	{
		$link = '/workspace/view?path=';
		$links = anchor($link, $this->CI->file_system->get_root());

		if (!$this->is_root())
		{
			$path = explode('/', $this->path);

			for ($i = 0; $i < count($path); $i++)
			{
				if ($i)
				{
					$link .= '/'.$path[$i];
				}
				else
				{
					$link .= $path[$i];
				}
				$links .= '/&#x200b<wbr>'.anchor($link, $path[$i]);
			}
		}
		return $links;
	}

	public function create_dir($name)
	{
		$this->CI->file_system->make_dir($name);
	}

	public function create_case($name)
	{
		$this->CI->file_system->exec_util('create_case', $name);
	}

	public function copy_case($name, $from)
	{
		$this->CI->file_system->exec_util('copy_case', $name.' '.$from);
	}

	public function delete()
	{
		$this->CI->file_system->delete();
	}

	public function check_dir_name_char($name)
	{
		return strpos($name, ' ') === FALSE && strpos($name, '/') === FALSE;
	}

	public function check_dir_name_start($name)
	{
		return $name[0] != '.';
	}

	public function check_dir_uniq($path)
	{
		return !$this->CI->file_system->is_dir($path);
	}

	public function redirect($function, $path = NULL)
	{
		redirect($this->make_url($function, $path));
	}

	public function make_url($function, $path = NULL, $full_path = FALSE)
	{
		return $function.$this->append_url($path, $full_path);
	}

	public function reload($parent = FALSE)
	{
		redirect('workspace/view'.$this->append_url($parent ? '..' : NULL));
	}

	public function append_url($path = NULL, $full_path = FALSE)
	{
		if (!$full_path)
		{
			if ($path == '')
			{
				$path = $this->path;
			}
			elseif ($path == '..')
			{
				$path = $this->parent_path;
			}
			else
			{
				$path = $this->append_path($path);
			}
		}

		return '?path='.$path;
	}

	public function append_path($path)
	{
		if ($this->path === '')
		{
			return $path;
		}

		return $this->path.'/'.$path;
	}
}

