<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Control_input extends Foam_input
{
	private $walls;
	private $cfields;

	public function __construct()
	{
		parent::__construct();

		$this->CI->load->config('control');
		$this->CI->load->library('control_db', NULL, 'db');

		$case = $this->CI->c_case->get_case();
		$this->walls = array();
		foreach ($case['boundaries'] as $boundary => $info)
		{
			if ($info['type'] == 'wall')
			{
				array_push($this->walls, $boundary);
			}
		}
	}

	public function submit_form()
	{
		$data = array();

		$data['control'] = array();

		foreach ($this->CI->config->item('input_items') as $key => $item)
		{
			$data['control'][$key] = $this->get_form_data('control_'.$key);
		}

		$keys = $this->get_form_data('forces');
		for ($i = 0; $i < count($keys); $i++)
		{
			$forces = array();
			$forces['type'] = 'forces';
			$forces['patches'] = $this->get_form_data('forces_patches_'.$keys[$i]);
			$forces['rhoInf'] = $this->get_form_data('forces_rhoInf_'.$keys[$i]);
			$forces['CofR'] = $this->get_form_data('forces_CofR_'.$keys[$i], 'vector');

			$data['functions']['forces'.($i + 1)] = $forces;
		}

		$keys = $this->get_form_data('forceCoeffs');
		for ($i = 0; $i < count($keys); $i++)
		{
			$forces = array();
			$forces['type'] = 'forceCoeffs';
			$forces['patches'] = $this->get_form_data('forceCoeffs_patches_'.$keys[$i]);
			$forces['rhoInf'] = $this->get_form_data('forceCoeffs_rhoInf_'.$keys[$i]);
			$forces['CofR'] = $this->get_form_data('forceCoeffs_CofR_'.$keys[$i], 'vector');
			$forces['liftDir'] = $this->get_form_data('forceCoeffs_liftDir_'.$keys[$i], 'vector');
			$forces['dragDir'] = $this->get_form_data('forceCoeffs_dragDir_'.$keys[$i], 'vector');
			$forces['pitchAxis'] = $this->get_form_data('forceCoeffs_pitchAxis_'.$keys[$i], 'vector');
			$forces['magUInf'] = $this->get_form_data('forceCoeffs_magUInf_'.$keys[$i]);
			$forces['lRef'] = $this->get_form_data('forceCoeffs_lRef_'.$keys[$i]);
			$forces['Aref'] = $this->get_form_data('forceCoeffs_Aref_'.$keys[$i]);

			$data['functions']['forceCoeffs'.($i + 1)] = $forces;
		}

		$this->CI->db->save_input_data($data);
	}

	public function get_js()
	{
		$js = 'var forceNum = 0;';
		$js .= 'function addForce() { jQuery(\'#force_items\').append(\'<div id="force\' + forceNum + \'"><input type="hidden" name="forces[]" value="\' + forceNum + \'"><table class="table-form">';
		$js .= '<tr><th style="width:240px"><h3>forces</h3></th><td class="text-right"><button type="button" class="btn btn-default btn-sm" onclick="removeItem(\\\'force\' + forceNum + \'\\\');">Remove</button>\'';
		$js .= " + checksInput('Patches', 'forces', 'patches_' + forceNum, jQuery.parseJSON('".json_encode($this->walls)."'), 4)";
		$js .= " + scalarInput('Density (kg/m3)', 'forces', 'rhoInf_' + forceNum, 1.2)"; 
		$js .= " + vectorInput('CofR', 'forces', 'CofR_' + forceNum, [0, 0, 0])";
		$js .= ' + \'</table><br/></div>\'); forceNum++; }';
		$js .= 'function addForceCoeff() { jQuery(\'#force_items\').append(\'<div id="force\' + forceNum + \'"><input type="hidden" name="forceCoeffs[]" value="\' + forceNum + \'"><table class="table-form">';
		$js .= '<tr><th style="width:240px"><h3>forces</h3></th><td class="text-right"><button type="button" class="btn btn-default btn-sm" onclick="removeItem(\\\'force\' + forceNum + \'\\\');">Remove</button>\'';
		$js .= " + checksInput('Patches', 'forceCoeffs', 'patches_' + forceNum, jQuery.parseJSON('".json_encode($this->walls)."'), 4)";
		$js .= " + scalarInput('Density (kg/m3)', 'forceCoeffs', 'rhoInf_' + forceNum, 1.2)"; 
		$js .= " + scalarInput('Reference Area (m2)', 'forceCoeffs', 'Aref_' + forceNum, 1)";
		$js .= " + scalarInput('lRef', 'forceCoeffs', 'lRef_' + forceNum, 1)";
		$js .= " + scalarInput('Reference Velocity (m/s)', 'forceCoeffs', 'magUInf_' + forceNum, 1)";
		$js .= " + vectorInput('CofR', 'forceCoeffs', 'CofR_' + forceNum, [0, 0, 0])";
		$js .= " + vectorInput('Drag Dir', 'forceCoeffs', 'dragDir_' + forceNum, [1, 0, 0])";
		$js .= " + vectorInput('Lift Dir', 'forceCoeffs', 'liftDir_' + forceNum, [0, 0, 1])";
		$js .= " + vectorInput('Pitch Axis', 'forceCoeffs', 'pitchAxis_' + forceNum, [0, 0, 1])";
		$js .= " + '</table><br/></div>'); forceNum++; }";
		$js .= "$(document).ready(function() { initFields(jQuery.parseJSON('".json_encode($this->rfields)."'), cfields = jQuery.parseJSON('".json_encode($this->cfields)."')); $('form').submit(function(event) { if (!validate()) { event.preventDefault(); } }); });";

		return $js;
	}

	protected function gen_inputs($values)
	{
		$this->inputs = array('control' => array(), 'force_functions' => array());
		$this->rfields = array('forces' => array(), 'forceCoeffs' => array());
		$this->cfields = array('forces' => array(), 'forceCoeffs' => array());

		foreach ($this->CI->config->item('input_items') as $key => $item)
		{
			if (db_element($key, $values) !== NULL)
			{
				$this->gen_input($key, $item, $values);
			}
		}

		$functions = db_data('functions', $values);
		if (!empty($functions))
		{
			$this->inputs['force_functions'] = array();
			$num = 0;

			foreach (db_data('functions', $values) as $name => $force)
			{
				$inputs = array();
				$type = db_entry('type', $force);
				$this->inputs['force_functions'][$num]['type'] = $type;

				switch ($type)
				{
					case 'forces':
						$this->make_force_patches_inputs($type, $num, $force);
						$this->add_force_input_scalar($type, $num, 'Density (kg/m3)', 'rhoInf', $force);
						$this->add_force_input_vector($type, $num, 'CofR', 'CofR', $force);

						break;
					case 'forceCoeffs':
						$this->make_force_patches_inputs($type, $num, $force);
						$this->add_force_input_scalar($type, $num, 'Density (kg/m3)', 'rhoInf', $force);
						$this->add_force_input_scalar($type, $num, 'Reference Area (m2)', 'Aref', $force);
						$this->add_force_input_scalar($type, $num, 'lRef', 'lRef', $force);
						$this->add_force_input_scalar($type, $num, 'Reference Velocity (m/s)', 'magUInf', $force);
						$this->add_force_input_vector($type, $num, 'CofR', 'CofR', $force);
						$this->add_force_input_vector($type, $num, 'Drag Dir', 'dragDir', $force);
						$this->add_force_input_vector($type, $num, 'Lift Dir', 'liftDir', $force);
						$this->add_force_input_vector($type, $num, 'Pitch Axis', 'pitchAxis', $force);

						break;
				}

				$num++;
			}
		}
	}

	private function gen_input($key, $item, $values)
	{
		switch ($item['type'])
		{
			case 'select':
				$this->add_input_select('control', $item['name'], $key, $values, $item['options']);
				break;
			case 'num':
				$this->add_input_scalar('control', $item['name'], $key, $values);
				break;
		}
	}

	private function make_force_patches_inputs($type, $num, $value)
	{
		$cols = 4;
		$width = 100 / $cols;
		$id = $type.'_patches__'.$num;
		$values = foam_data($this->extract_element('patches', $value));

		$input = '<table style="width:100%">';
		for ($i = 0; $i < count($this->walls); $i++)
		{
			if ($i % $cols == 0) $input .= '<tr>';
			$input .= '<td style="width:'.$width.'%"><input type="checkbox" name="'.$id.'[]" id="'.$id.'" value="'.$this->walls[$i].'"'.(array_search($this->walls[$i], $values) === FALSE ? '' : ' checked').'> '.$this->walls[$i].'</td>';
			if ($i % $cols == $cols - 1) $input .= '</tr>';
		}
		$input .= '</table>';

		$this->inputs['force_functions'][$num]['Patches'] = $input;

		$this->cfields[$type][$id] = 'Patches';
	}

	private function add_force_input_scalar($type, $num, $name, $key, $values)
	{
		$id = $type.'_'.$key.'__'.$num;
		$this->inputs['force_functions'][$num][$name] = make_input_scalar($id, foam_scalar($this->extract_element($key, $values)));
		$this->rfields[$type][$id] = $name;
	}
	
	private function add_force_input_vector($type, $num, $name, $key, $values)
	{
		$id = $type.'_'.$key.'__'.$num;
		$this->inputs['force_functions'][$num][$name] = make_input_vector($id, foam_data($this->extract_element($key, $values)));

		$this->rfields[$type][$id.'_x'] = $name;
		$this->rfields[$type][$id.'_y'] = $name;
		$this->rfields[$type][$id.'_z'] = $name;
	}
}
