<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Decompose_par_input extends Foam_input
{
	public function __construct($params = NULL)
	{
		parent::__construct();

		$this->CI->load->library('decompose_par_db', $params, 'db');
	}

	public function submit_form()
	{
		$data['numberOfSubdomains'] = $this->get_form_data('numberOfSubdomains');
		$data['method'] = $this->get_form_data('method');

		$this->CI->db->save_input_data($data);
		$this->save_hostfile($this->get_hosts_data());
	}

	protected function gen_inputs($values)
	{
		$this->add_input_int('', 'numberOfSubdomains', 'numberOfSubdomains', $values, 'min="2"', 2);
		$this->add_input_select('', 'method', 'method', $values, array('scotch', 'metis'));

		$this->make_hosts_input();
	}

	private function make_hosts_input()
	{
		$nodes = $this->CI->file_system->get_nodes();

		if (!$nodes)
		{
			return;
		}

		$buffer = $this->CI->file_system->cat('hostfile');
		$hosts = array();
		if (!empty($buffer))
		{
			foreach ($buffer as $line)
			{
				$line = trim(preg_replace('/\s+/', ' ',$line));
				$host = explode(' ', $line);

				if (!empty($host))
				{
					$cpu = '';

					if (count($host) > 1)
					{
						$buf = explode('=', $line);
						if (count($buf) > 1)
						{
							$cpu = $buf[1];
						}
					}

					$hosts[$host[0]] = $cpu;
				}
			}
		}

		$input = '<table style="width:100%">';
		foreach ($nodes as $node => $ncpu)
		{
			$input .= $this->make_input_host($node, element($node, $hosts), $ncpu);
		}
		$input .= '</table>';

		$this->inputs['']['hosts'] = $input;
	}

	private function make_input_host($node, $cpu, $ncpu)
	{
		$checkbox_name = "host_".$node;
		$checked = $cpu ? ' checked' : '';
		$value = $cpu > 0 ? $cpu : '';
		return '<tr><td style="width:160px"><input type="checkbox" name="'.$checkbox_name.'" value="use"'.$checked.'> <label for="'.$checkbox_name.'">'.$node.'</label></td><th style="width:30px">cpu</th><td>'.make_input_int('cpu_'.$node, $value, 'min="1" max="'.$ncpu.'"').'</td></tr>';
	}

	private function get_hosts_data()
	{
		$nodes = $this->CI->file_system->get_nodes();
		$data = '';
		foreach ($nodes as $node => $ncpu)
		{
			if ($this->get_form_data('host_'.$node) == 'use')
			{
				$cpu = $this->get_form_data('cpu_'.$node);
				$cpu = $cpu ? ' cpu='.$cpu : '';
				$data .= $node.$cpu."\n";
			}
		}

		return $data;
	}

	private function save_hostfile($hosts)
	{
		if (empty($hosts))
		{
			$this->CI->file_system->remove_file('hostfile');
		}
		else
		{
			$this->CI->file_system->write_file('hostfile', $hosts);
		}
	}
}
