<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Dynamic_mesh_input extends Foam_input
{
	public function __construct()
	{
		parent::__construct();

		$this->CI->load->config('dynamic_mesh', TRUE);
		$this->CI->load->library('dynamic_mesh_db', NULL, 'db');
	}

	public function submit_form()
	{
		if ($_FILES['time_data_file']['name'] != '')
		{
			$this->CI->c_case->upload('constant', 'time_data_file');
		}

		$data = array();

		$data['dynamicFvMesh'] = $this->get_form_data('dynamicFvMesh');
		$data['solidBodyMotionFvMeshCoeffs'] = $this->get_form_solid_body_motion_fv_mesh_coeffs();

		$this->CI->db->save_input_data($data);
	}

	public function get_js()
	{
		$js = "$(document).ready(function() { $('#browse').click(function() { $('#time_data_file').click(); }); ";
		$js .= "$('#time_data_file').change(function() { $('#timeDataFileName').val($(this).prop('files')[0]['name']); });";
		$js .= "$('form').submit(function(event) { if ($('#time_data_file').val() == '' & $('#timeDataFileName').val() == '') { alert('Select Time Data File'); event.preventDefault(); } }); });";

		return $js;
	}

	protected function gen_inputs($values)
	{
		$config = $this->CI->config->item('dynamic_mesh');
		
		$this->add_input_select('', 'dynamicFvMesh', 'dynamicFvMesh', $values, $config['options']['dynamicFvMesh']);

		$solidBodyMotionFvMeshCoeffs = $this->extract_element('solidBodyMotionFvMeshCoeffs', $values);
		$this->add_input_select('', 'solidBodyMotionFunction', 'solidBodyMotionFunction', $this->extract_element('solidBodyMotionFvMeshCoeffs', $values), $config['options']['solidBodyMotionFunction']);

		$tabulated6DoFMotionCoeffs = $this->extract_element('tabulated6DoFMotionCoeffs', $solidBodyMotionFvMeshCoeffs);
		$this->add_input_vector('', 'CofG', 'CofG', $tabulated6DoFMotionCoeffs);

		$timeDataFileName = foam_entry($this->extract_element('timeDataFileName', $tabulated6DoFMotionCoeffs));
		$this->inputs['']['Time Data File'] = '<table style="width:100%"><tr><td><input type="text" name="timeDataFileName" id="timeDataFileName" class="form-control" value="'.substr($timeDataFileName, 21, strlen($timeDataFileName) - 21).'" readonly /></td><td class="text-right" style="width:70px"><button type="button" name="browse" id="browse" class="btn btn-sm btn-default">Browse</button></td></tr></table><input type="file" name="time_data_file" id="time_data_file" style="display:none" />';
	}

	private function get_form_solid_body_motion_fv_mesh_coeffs()
	{
		$data = array();

		$data['solidBodyMotionFunction'] = $this->get_form_data('solidBodyMotionFunction');
		$data['tabulated6DoFMotionCoeffs'] = $this->get_form_tabulated_6_dot_f_motion_coeffs();

		return $data;
	}

	private function get_form_tabulated_6_dot_f_motion_coeffs()
	{
		$data['CofG'] = $this->get_form_data('CofG', 'vector');
		$data['timeDataFileName'] = $this->get_form_data('timeDataFileName');

		return $data;
	}
}
