<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class File_system
{
	protected $CI;

	private $server;
	private $root;
	private $dir;
	private $cmd;
	private $is_case;
	private $data_dir;

	public function __construct($params)
	{
		$this->CI =& get_instance();
		
		$path = $this->CI->input->get('path');
		$user = $params['user'];

		$this->CI->load->config('foam_server', TRUE);
		$this->server = $this->CI->config->item('foam_server')['localhost'];
		if (empty($this->server))
		{
			show_error("foam_server configuration was not found", 404, "Invalid Request");
		}

		$this->cmd = 'sudo -u '.$user.' ';
		$this->root = $this->server['home_dir'].DIRECTORY_SEPARATOR.$user.DIRECTORY_SEPARATOR.$this->server['baram_dir'];

		if ($path === '')
		{
			$this->dir = $this->root;
		}
		else
		{
			$this->dir = $this->root.DIRECTORY_SEPARATOR.$path;
		}

		if ($this->is_dir('.case'))
		{
			$this->is_case = TRUE;
			$this->data_dir = '.case/';
		}
	}

	public function is_case()
	{
		return $this->is_case;
	}

	public function is_dir($path = NULL)
	{
		$res = $this->exec_util('check_dir', $path, TRUE);
		if ($res === FALSE)
		{
			return NULL;
		}
		
		return $res[0] === '0';
	}

	public function get_root()
	{
		return $this->root;
	}

	public function get_nodes()
	{
		return $this->server['nodes'];
	}

	public function get_baram_dir()
	{
		return $this->server['baram_dir'];
	}

	public function get_path($path = NULL)
	{
		if ($path === NULL)
		{
			return $this->dir;
		}
		elseif ($path == '..')
		{
			if ($this->is_root())
			{
				return FALSE;
			}

			return substr($this->dir, 0, strrpos($this->dir, DIRECTORY_SEPARATOR)); 
		}

		return $this->dir.DIRECTORY_SEPARATOR.$path;
	}

	public function get_data_file($name)
	{
		return $this->data_dir.$name;
	}

	public function get_files($path = '.')
	{
		$dirs = array();
		$cases = array();
		$files = array();

		$list = $this->exec_util('read_dir', $path);
		
		foreach ($list as $info)
		{
			switch ($info[0])
			{
				case 'd':
					array_push($dirs, substr($info, 2));
					break;
				case 'c':
					array_push($cases, substr($info, 2));
					break;
				case 'f':
					array_push($files, substr($info, 2));
					break;
			}
		}

		return array('dirs' => $dirs, 'cases' => $cases, 'files' => $files);
	}

	public function delete()
	{
		$this->remove_dir();
	}

	public function write_file($file_name, $contents)
	{
		$file = fopen($this->get_path($file_name), 'w');
		if (!$file)
		{
			show_error('Fail to open file : '.$file_name);
		}

		fwrite($file, $contents);
		fclose($file);
	}

	public function read_file($path)
	{
		$file_path = $this->get_path($path);

		if (is_file($file_path))
		{
			return @file_get_contents($file_path);
		}

		return NULL;
	}

	public function write_data_file($file_name, $contents)
	{
		$this->write_file($this->get_data_file($file_name), $contents);
	}

	public function read_data_file($file_name)
	{
		return $this->read_file($this->get_data_file($file_name));
	}

	public function cat_data_file($file_name)
	{
		return $this->cat($this->get_data_file($file_name));
	}

	public function save_data($name, $value)
	{
		$file = fopen($this->get_path($this->data_dir.$name), 'w');
		if (!$file)
		{
			show_error('Fail to save working data');
		}

		fwrite($file, serialize($value));
		fclose($file);
	}

	public function load_data($name)
	{
		$data = $this->read_file($this->data_dir.$name);

		if ($data === NULL)
		{
			return NULL;
		}

		return unserialize($data);
	}

	public function is_file($path)
	{
		return is_file($this->get_path($path));
	}

	public function tail($path, $line_num)
	{
		return $this->shell_exec('tail -n '.$line_num.' '.$this->get_path($path));
	}

	public function cat($path)
	{
		return $this->exec_cmd('cat '.$this->get_path($path), TRUE);
	}

	public function find_files()
	{
		return $this->exec_util('find_files');
	}

	public function compress($name)
	{
		$filename = $name.'.tar.gz';
		
		$this->exec_util('compress', $name.' '.$this->root)[0];

		return $this->root.DIRECTORY_SEPARATOR.$filename;
	}
	
	public function exec_foam($cmd)
	{
		exec($this->cmd.$this->server['util_dir'].DIRECTORY_SEPARATOR.'exec_foam '.$this->dir.' '.$cmd, $output, $return);

		if ($return)
		{
			show_error($cmd.'<br>'.implode('<br />', $output));
		}

		return $return;
	}
	
	public function is_root()
	{
		return $this->dir == $this->root;
	}

	public function remove_file($path)
	{
		return $this->exec_cmd("rm -f ".$this->get_path($path));
	}

	public function upload($path, $file)
	{
		$this->CI->load->library('upload', array('upload_path' => $this->get_path($path), 'allowed_types' => '*', 'overwrite' => TRUE));

		if (!$this->CI->upload->do_upload($file))
		{
			show_error($this->CI->upload->display_errors());
		}
	}

	public function exec_util($util, $args = '', $check = FALSE)
	{
		return $this->exec_cmd($this->server['util_dir'].DIRECTORY_SEPARATOR.$util.' '.$this->dir.' '.$args, $check);
	}

	public function empty_dir($path)
	{
		return $this->exec_cmd('rm -rf '.$this->get_path($path).DIRECTORY_SEPARATOR.'*');
	}

	public function make_dir($path)
	{
		return $this->exec_cmd("mkdir -m 775 ".$this->get_path($path));
	}

	public function make_data_dir()
	{
		if (!$this->is_dir($this->data_dir))
		{
			$this->make_dir($this->data_dir);
		}
	}

	private function exec_cmd($cmd, $check = FALSE)
	{
		exec('bash -c "'.$this->cmd.$cmd.'" 2>&1', $output, $return);

		if ($return)
		{
			if ($check)
			{
				return FALSE;
			}

			show_error($cmd.'<br>'.implode('<br />', $output));
		}

		return $output;
	}

	private function shell_exec($cmd)
	{
		return shell_exec($this->cmd.$cmd);
	}

	private function remove_dir($path = NULL)
	{
		return $this->exec_cmd("rm -rf ".$this->get_path($path));
	}
}
