<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Foam_db
{
	protected $CI;

	protected $buffer;
	protected $error;
	protected $header;
	protected $data;

	private $temp;
	private $read;
	private $char;
	private $current;
	private $close_brace;

	private $braces;
	private $delimeters;

	private $file_path;
	private $file_name;
	private $file;

	public function __construct($params = NULL)
	{
		$this->CI =& get_instance();

		if (empty($params['list_off']))
		{
			$this->delimeters = " \t\n\r{}();";
			$this->braces = "{(;";
		}
		else
		{
			$this->delimeters = " \t\n\r{};";
			$this->braces = "{;";
		}

		if (!empty($params))
		{
			$this->file_path = element('file_path', $params);
		}

		$this->temp[0] = array(PS_TYPE => NULL, 'close_brace' => '');
		$this->current = 0;
	}

	public function get_error()
	{
		return $this->error;
	}

	public function init($config)
	{
		$this->header = array(
			'version'	=> $this->CI->config->item('foamfile_version'),
			'format'	=> 'ascii',
			'class'		=> $config['class'],
			'location'	=> '"'.$config['location'].'"',
			'object'	=> $config['object']
		);

		$this->file_name = $config['object'];
		if ($this->file_path === NULL)
		{
			$this->file_path = $config['location'].'/'.$config['object'];
		}
	}

	public function load()
	{
		$this->buffer = $this->CI->file_system->read_file($this->file_path);

		if ($this->buffer === NULL)
		{
			$this->set_error(1, "Fail to read file ".$this->CI->file_system->get_path($this->file_path));

			$this->buffer = $this->CI->file_system->read_data_file($this->file_name);
			if ($this->buffer !== NULL)
			{
				$data = $this->parse();
				return $data;
			}

			return FALSE;
		}

		$data = $this->parse();

		return $data;
	}

	public function save($input = TRUE)
	{
		$this->file = @fopen($this->CI->file_system->get_path($this->file_path), 'w');
		if (!$this->file)
		{
			show_error("Fail to write input data file '".$this->file_path."'");
		}

		$this->write_header();
		if ($input)
		{
			$this->write_input_data();
		}
		else
		{
			$this->write($input);
		}

		@fclose($this->file);
	}

	public function save_input_data($data)
	{
		$this->data = $data;
		$this->save();
	}

	protected function set_error($code, $msg)
	{
		$this->error['code'] = $code;
		$this->error['msg'] = $msg;

		return FALSE;
	}

	protected function parse()
	{
		$this->read = 0;

		$format = $this->read_format();
		while ($format)
		{
			switch ($format[PS_TYPE])
			{
				case ENTITY_DICTIONARY:
				case ENTITY_LIST:
					$this->parse_open($format);
					break;
				case ENTITY_ENTRIES:
					{
						$this->set_data($format['keyword'], $format[PS_TYPE], NULL, $format['entries']);
					}
					break;
			}

			$format = $this->read_format();
		}

		if ($this->temp[$this->current][PS_TYPE] && empty($this->error))
		{
			return $this->set_error(2, "Fail to parse ".$this->data['keyword']." : can't find close brace '".$this->close_brace);
		}
		$data = $this->temp[$this->current];
		unset($this->temp);

		return $data;
	}

	protected function remain_read()
	{
		return $this->read < strlen($this->buffer);
	}

	protected function skip($chars = "")
	{
		$skip_chars = " \t\n".$chars;

		while ($this->remain_read())
		{
			$char2 = substr($this->buffer, $this->read, 2);
			while ($this->remain_read() && strpos($skip_chars, $char2[0]) !== FALSE)
			{
				$this->read_char();
				$char2 = substr($this->buffer, $this->read, 2);
			}

			switch ($char2)
			{
				case "//":
					$pos = strpos($this->buffer, "\n", $this->read);
					if ($pos)
					{
						$this->read = $pos + 1;
					}
					else
					{
						$this->read = strlen($this->buffer);
					}
					break;
				case "/*":
					$pos = strpos($this->buffer, "*/", $this->read);
					if ($pos)
					{
						$this->read = $pos + 2;
					}
					else
					{
						$this->read = strlen($this->buffer);
						return $this->set_error(11, "Unexpected EOF with Unclosed comments");
					}
					break;
				default:
					return;
			}
		}
	}

	protected function read_char()
	{
		if ($this->remain_read())
		{
			$this->char = $this->buffer[$this->read];
			$this->read++;
			return $this->char;
		}

		return FALSE;
	}

	protected function read_token($delimeters)
	{
		$token = FALSE;
		
		$this->skip();

		while ($this->read_char() !== FALSE)
		{
			if (($pos = strpos($delimeters, $this->char)) !== FALSE)
			{
				return $token;
			}

			$token .= $this->char;

			$char2 = substr($this->buffer, $this->read, 2);
			if ($char2 == '//' || $char2 == '/*')
			{
				return $token;
			}
		}

		return $token;
	}

	protected function read_format()
	{
		$tokens = array();
		$i = 0;

		while ($this->remain_read())
		{
			$token = $this->read_token($this->delimeters);

			if ($token !== FALSE)
			{
				$tokens[$i] = $token;
				$i++;
			}

			if (strpos($this->braces, $this->char) !== FALSE)
			{
				return $this->format($this->char, $tokens);
			}

			if ($this->char == $this->close_brace)
			{
				$this->parse_close(empty($tokens) ? FALSE : $tokens);
				$tokens = array();
				$i = 0;
				$this->skip(';');
			}
		}

		if ($i)
		{
			return $this->set_error(30, "Unexpected EOF during parsing '".$tokens[1]."'");
		}

		return FALSE;
	}

	protected function read_block($format)
	{
		$this->CI->log("block start");

		$start_pos = $this->read;
		$this->skip();
		$depth = 1;
		$open_brace = $format[PS_TYPE];
		$close_brace = $format['close_brace'];

		while ($this->read_char() !== FALSE)
		{
			switch ($this->char)
			{
				case $open_brace:
					$depth++;
					break;
				case $close_brace:
					$depth--;
					break;
			}

			if ($depth == 0)
			{
				break;
			}

			$this->skip();
		}

		if ($depth > 0)
		{
			return $this->set_error(22, "Fail to parse ".$format['keyword']." : can'f ind close brace '".$close_brace);
			return FALSE;
		}

		$this->CI->log("block end : [".$start_pos."~".$this->read."] ".substr($this->buffer, $start_pos, $this->read - $start_pos)."");
		return substr($this->buffer, $start_pos, $this->read - $start_pos - 2);
	}
	
	protected function write_empty_line()
	{
		fwrite($this->file, "\n");
	}

	protected function write_line($line, $indent = 0)
	{
		fwrite($this->file, str_repeat(' ', $indent * 4));
		fwrite($this->file, $line);
		fwrite($this->file, "\n");
	}

	protected function write_entry($name, $value, $indent, $padding)
	{
		if ($value !== NULL)
		{
			fwrite($this->file, str_repeat(' ', $indent * 4));
			fwrite($this->file, str_pad($name, $padding));
			fwrite($this->file, "\t");
			fwrite($this->file, $value);
			fwrite($this->file, ';');
			$this->write_empty_line();
		}
	}

	protected function write_list($name, $list, $indent, $padding)
	{
		$this->write_line($name, $indent);
		$this->write_line('(', $indent);
		foreach ($list as $value)
		{
			$this->write_line($value, $indent + 1, $padding);
		}
		$this->write_line(');', $indent);
	}

	protected function write_dictionary($name, $dictionary, $indent = 0, $padding = 0)
	{
		$this->write_line($name, $indent);
		$this->write_line('{', $indent);
		foreach ($dictionary as $key => $value)
		{
			$this->write_entry($key, $value, $indent + 1, $padding);
		}
		$this->write_line('}', $indent);
	}

	protected function write_input_data()
	{
		foreach ($this->data as $key => $value)
		{
			$this->write_input_item($key, $value);
			$this->write_empty_line();
		}

		$this->write_hr_line();
	}

	protected function write_input_item($key, $value, $indent = 0, $padding = 12)
	{
		$item = $this->CI->config->item('input_items')[$key];

		switch ($item['type'])
		{
			case 'dimset':
				$this->write_entry($key, $this->make_dimset_value($value, $item['ref']), $indent, $padding);
				break;
			default:
				$this->write_entry($key, $value, $indent, $padding);
				break;
		}
	}

	protected function write_hr_dash()
	{
		$this->write_line('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //');
	}

	protected function write_hr_line()
	{
		$this->write_line('// ************************************************************************* //');
	}

	protected function make_dimset_value($value, $ref)
	{
		foreach ($this->CI->config->item('dimset_attrs')[$ref] as $key => $attrs)
		{
			if ($this->CI->c_case->check_condition($attrs))
			{
				return $attrs['dim'].' '.$value;
			}
		}

		show_error("Write format for '".$ref."' not found");
	}

	private function get_file_path()
	{
		return $this->file_path;
	}

	private function format($type, $tokens)
	{
		$cnt = count($tokens);
		if ($cnt)
		{
			$keyword = $tokens[0];
		}
		$entries = NULL;

		switch ($type)
		{
			case ENTITY_DICTIONARY:
				if ($cnt != 1)
				{
					return $this->set_error(31, "Wrong keyword of dictionary : ".implode(" ", $tokens));
				}
				$close_brace = '}';
				break;
			case ENTITY_LIST:
				switch ($cnt)
				{
					case 1:
						if (is_numeric($tokens[0]))
						{
							$keyword = '';
							$entries = array(NULL, $tokens[0]);
						}
						break;
					case 2:
						if (is_numeric($tokens[1]))
						{
							$entries = array(NULL, $tokens[1]);
						}
						else
						{
							$entries = array($tokens[1], NULL);
						}
						break;
					case 3:
						$entries = array($tokens[1], $tokens[2]);
						break;
					default:
						return $this->set_error(32, "Wrong keyword of list : ".implode(" ", $tokens));
						break;
				}
				$close_brace = ')';
				break;
			case ENTITY_ENTRIES:
				$entries = array_slice($tokens, 1);
				$close_brace = '';
				break;
		}

		return array(PS_TYPE => $type, 'keyword' => $keyword, 'entries' => $entries, 'close_brace' => $close_brace, PS_DATA => array());
	}

	private function parse_open($format)
	{
		$this->current++;
		$this->temp[$this->current] = $format;
		$this->close_brace = $format['close_brace'];
	}

	private function parse_close($data = FALSE)
	{
		if ($this->current)
		{
			$result = $this->temp[$this->current];
			$this->current--;
			$this->set_data($result['keyword'], $result[PS_TYPE], $result['entries'], $data === FALSE ? $result[PS_DATA] : $data);
			$this->close_brace = $this->temp[$this->current]['close_brace'];

			return TRUE;
		}
			
		return $this->set_error(41, "Unexpected '".$this->char."'");
	}

	private function set_data($keyword, $type, $info, $data)
	{
		$this->temp[$this->current][PS_DATA][$keyword] = array(PS_TYPE => $type, PS_INFO => $info, PS_DATA => $data);
	}

	private function write_header()
	{
		$this->write_line('/*--------------------------------*- C++ -*----------------------------------*\\');
		$this->write_line('| =========                 |                                                 |');
		$this->write_line('| \\\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |');
		$this->write_line('|  \\\\    /   O peration     | Version:  '.$this->CI->config->item('openfoam_version').'                                 |');
		$this->write_line('|   \\\\  /    A nd           | Web:      www.OpenFOAM.org                      |');
		$this->write_line('|    \\\\/     M anipulation  |                                                 |');
		$this->write_line('\*---------------------------------------------------------------------------*/');

		$this->write_dictionary('FoamFile', $this->header, 0, 8);
		$this->write_hr_dash();
		$this->write_empty_line();
	}
}
