<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Foam_input
{
	protected $inputs;
	protected $rfields;
	protected $error;

	public function __construct()
	{
		$this->CI =& get_instance();
		$this->CI->load->library('foam_db');
		$this->CI->load->helper('input');
	}

	public function load()
	{
		$this->inputs= array();
		$this->rfields = array();

		$data = $this->CI->db->load();
		$this->error = $this->CI->db->get_error();
		$this->gen_inputs($data);

		return array('error' => $this->error, 'inputs' => $this->inputs, 'rfields' => $this->rfields);
	}

	public function get_js()
	{
		return "$(document).ready(function() { rfields = jQuery.parseJSON('".json_encode($this->rfields)."'); $('form').submit(function(event) { if (!validate()) { event.preventDefault(); } }); });";
	}

	protected function gen_inputs($values)
	{
		$this->inputs = $values;
	}

	protected function add_input_select($category, $name, $key, $values, $options, $optional = FALSE)
	{
		$id = empty($category) ? $key : $category.'_'.$key;
		$this->add_input($category, $name, $id, make_input_select($id, foam_entries($this->extract_element($key, $values, $optional)), $options), FALSE);
	}
	
	protected function add_input_vector($category, $name, $key, $values, $default = array(0, 0, 0), $optional = FALSE)
	{
		$id = empty($category) ? $key : $category.'_'.$key;
		$this->add_input($category, $name, $id, make_input_vector($id, foam_data($this->extract_element($key, $values, $optional), $default), $optional), FALSE);

		if (!$optional)
		{
			$this->rfields[$category][$id.'_x'] = $name;
			$this->rfields[$category][$id.'_y'] = $name;
			$this->rfields[$category][$id.'_z'] = $name;
		}
	}

	protected function add_input_scalar($category, $name, $key, $values, $default = NULL, $optional = FALSE)
	{
		$id = empty($category) ? $key : $category.'_'.$key;
		$this->add_input($category, $name, $id, make_input_scalar($id, foam_scalar($this->extract_element($key, $values, $optional), $default), $optional), !$optional);
	}

	protected function add_input_int($category, $name, $key, $values, $attrs = NULL, $default = NULL, $optional = FALSE)
	{
		$id = empty($category) ? $key : $category.'_'.$key;
		$this->add_input($category, $name, $id, make_input_int($id, foam_scalar($this->extract_element($key, $values, $optional), $default), $attrs, $optional), !$optional);
	}

	protected function add_input_value($category, $name, $key, $values)
	{
		$id = empty($category) ? $key : $category.'_'.$key;
		$this->add_input($category, $name, $id, make_input_value($id, foam_entries($this->extract_element($key, $values))), FALSE);
	}

	protected function add_input($category, $name, $id, $input, $rfield = TRUE)
	{
		$this->inputs[$category][$name] = $input;

		if ($rfield)
		{
			$this->rfields[$category][$id] = $name;
		}
	}

	protected function get_form_data($key, $type = NULL)
	{
		switch ($type)
		{
			case 'vector':
				return '('.$this->CI->input->post($key.'_x').' '.$this->CI->input->post($key.'_y').' '.$this->CI->input->post($key.'_z').')';
			default:
				return $this->CI->input->post($key) == '' ? NULL : $this->CI->input->post($key);
		}
	}

	protected function extract_element($key, $values, $optional = FALSE)
	{
		$data = db_element($key, $values);

		if ($data)
		{
			return $data;
		}

		if (!$optional)
		{
			$this->set_error(200, $key.' is required');
		}

		return NULL;
	}

	protected function set_error($code, $msg)
	{
		if (!$this->error['code'])
		{
			$this->error['code'] = $code;
			$this->error['msg'] = $msg;
		}
	}
}
