<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Scalar_input extends Variable_input
{
	public function __construct($params)
	{
		parent::__construct();

		$this->init($params);
	}

	protected function make_input_internal_field($values)
	{
		$this->add_input_scalar('', 'Internal Field ('.$this->config['unit'].')', 'internalField', $values, $this->config['default']);
	}
	
	protected function make_boundary_inputs($boundary, $values, $type = 'zeroGradient')
	{
		$type = db_entry('type', $values, $type);

		$this->add_input($boundary, 'Type', $boundary.'_type', $this->make_input_types($boundary.'_type', $type), FALSE);

		switch ($type)
		{
			case 'calculated':
			case 'fixedValue':
				$this->add_input_scalar($boundary, 'Value ('.$this->config['unit'].')', 'value', $values);
				break;
			case 'empty':
			case 'kqRWallFunction':
			case 'omegaWallFunction':
			case 'symmetry':
			case 'symmetryPlane':
			case 'zeroGradient':
				break;
			case 'epsilonWallFunction':
			case 'nutkWallFunction':
				$this->add_input_scalar($boundary, 'Cmu', 'Cmu', $values, 0.09);
				$this->add_input_scalar($boundary, 'kappa', 'kappa', $values, 0.41);
				$this->add_input_scalar($boundary, 'E', 'E', $values, 9.8);
				break;
			case 'inletOutlet':
				$this->add_input_scalar($boundary, 'Inlet Value ('.$this->config['unit'].')', 'inletValue', $values);
				break;
			case 'totalPressure':
				$this->add_input_scalar($boundary, 'Gamma', 'gamma', $values);
				$this->add_input_scalar($boundary, 'p0', 'p0', $values);
				break;
		}
	}

	protected function get_form_field($name)
	{
		return $this->get_form_data($name);
	}

	protected function get_form_boundary($name)
	{
		$boundary = array();
		
		$boundary['type'] = $this->get_form_data($name.'_type');

		switch ($boundary['type'])
		{
			case 'calculated':
			case 'fixedValue':
				$boundary['value'] = 'uniform '.$this->get_form_data($name.'_value');
				break;
			case 'empty':
			case 'symmetry':
			case 'symmetryPlane':
			case 'zeroGradient':
				break;
			case 'epsilonStdWallFunction':
			case 'epsilonWallFunction':
			case 'nutkWallFunction':
			case 'nutStdWallFunction':
				$boundary['Cmu'] = $this->get_form_data($name.'_Cmu');
				$boundary['kappa'] = $this->get_form_data($name.'_kappa');
				$boundary['E'] = $this->get_form_data($name.'_E');
				$boundary['value'] = '$internalField';
				break;
			case 'inletOutlet':
				$boundary['inletValue'] = 'uniform '.$this->get_form_data($name.'_inletValue');
				break;
			case 'kqRWallFunction':
			case 'omegaWallFunction':
				$boundary['value'] = '$internalField';
				break;
			case 'totalPressure':
				$boundary['gamma'] = $this->get_form_data($name.'_gamma');
				$boundary['p0'] = $this->get_form_data($name.'_p0');
				break;
		}

		return $boundary;
	}
}
