<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Solution_input extends Foam_input
{
	private $boundary_inputs;

	public function __construct($params)
	{
		parent::__construct();

		$this->boundary_inputs = $params['boundary_inputs'];

		$this->CI->load->config('solution', TRUE);
		$this->CI->load->config('variable_conditions', TRUE);

		$this->CI->load->library('solution_db', NULL, 'db');
	}

	public function get_js()
	{
		$js = "$(document).ready(function() { rfields = jQuery.parseJSON('".json_encode($this->rfields)."'); $('form').submit(function(event) { if (!validate()) { event.preventDefault(); } });";
		$js .= "$(\"select[id$='_solver']\").change(function() { onChangeSolver($(this)); }); });";

		return $js;
	}

	public function submit_form()
	{
		$config = $this->CI->config->item('solution');
		$sconfig = $this->CI->config->item('solvers')[$this->CI->c_case->get_attr('solver')]['solution'];

		$data = array('solvers' => array(), 'solution' => array(), 'relaxationFactors' => array());

		foreach ($this->boundary_inputs as $variable => $info)
		{
			$vars = element($variable, $config['variables'][$sconfig['variables']]);
			if (!empty($vars))
			{
				foreach ($vars as $var => $type)
				{
					$data['solvers'][$var] = $this->get_form_solvers($variable);
					if ($type)
					{
						$data['relaxationFactors'][$type][$var] = $this->get_form_data('relaxationFactors_'.$var);
					}
				}
			}
		}

		$solutions = $this->CI->config->item('solvers')[$this->CI->c_case->get_attr('solver')]['solution'];
		foreach ($solutions['solution'] as $solution => $info)
		{
			$data['solution'][$solution] = $this->get_form_solution($solution, $info);
		}

		$this->CI->db->save_input_data($data);
	}

	protected function gen_inputs($values)
	{
		$config = $this->CI->config->item('solution');
		$sconfig = $this->CI->config->item('solvers')[$this->CI->c_case->get_attr('solver')]['solution'];

		$relFactors = array();
		$element = $this->extract_element('solvers', $values);
		foreach ($this->boundary_inputs as $variable => $info)
		{
			$vars = element($variable, $config['variables'][$sconfig['variables']]);
			if (!empty($vars))
			{
				foreach ($vars as $var => $type)
				{
					$this->gen_solver_inputs($var, $this->extract_element($var, $element), $config['solvers'][$var], $config['smoothers']);
					if ($type)
					{
						$relFactors[$var] = $type;
					}
				}
			}
		}

		$solutions = $this->CI->config->item('solvers')[$this->CI->c_case->get_attr('solver')]['solution'];
		foreach ($solutions['solution'] as $solution => $info)
		{
			$this->gen_solution_inputs($solution, $this->extract_element($solution, $values), $info);
		}

		$element = $this->extract_element('relaxationFactors', $values);
		foreach ($relFactors as $variable => $type)
		{
			$this->add_input_scalar('relaxationFactors', $variable, $variable, $this->extract_element($type, $element));
		}
	}

	private function gen_solver_inputs($variable, $values, $solvers, $smoothers)
	{
		$solver = foam_entry($this->extract_element('solver', $values), $solvers[0]);
		$this->add_input($variable, 'solver', $variable.'_solver', make_input_select($variable.'_solver', $solver, $solvers), FALSE);
		$this->add_input_scalar($variable, 'tolerance', 'tolerance', $values);
		$this->add_input_scalar($variable, 'relTol', 'relTol', $values);

		switch ($solver)
		{
			case 'GAMG':
				$this->add_input_select($variable, 'smoother', 'smoother', $values, $smoothers);
				$this->add_input_int($variable, 'nPreSweeps', 'nPreSweeps', $values);
				$this->add_input_scalar($variable, 'nPostSweeps', 'nPostSweeps', $values);
				$this->add_input_select($variable, 'cacheAgglomeration', 'cacheAgglomeration', $values, array('on', 'off'));
				$this->add_input_select($variable, 'agglomerator', 'agglomerator', $values, array('dummy', 'faceAreaPair'));
				$this->add_input_int($variable, 'nCellsInCoarsestLevel', 'nCellsInCoarsestLevel', $values);
				$this->add_input_int($variable, 'mergeLevels', 'mergeLevels', $values);
				break;
			case 'smoothSolver':
				$this->add_input_select($variable, 'smoother', 'smoother', $values, $smoothers);
				$this->add_input_int($variable, 'nSweeps', 'nSweeps', $values, NULL, NULL, TRUE);
				break;
		}
	}

	private function gen_solution_inputs($solution, $values, $info)
	{
		foreach ($this->CI->config->item('solution')[$info] as $key => $item)
		{
			$this->gen_solution_input($solution, $key, $item, $values);
		}
	}

	private function gen_solution_input($solution, $name, $item, $values)
	{
		switch ($item['type'])
		{
			case 'int':
				$this->add_input_int($solution, $name, $name, $values);
				break;
			case 'select':
				$this->add_input_select($solution, $name, $name, $values, $item['options']);
				break;
		}
	}

	private function get_form_solvers($name)
	{
		$solver = array();

		$solver['solver'] = $this->get_form_data($name.'_solver');

		switch ($solver['solver'])
		{
			case 'GAMG':
				$solver['smoother'] = $this->get_form_data($name.'_smoother');
				$solver['tolerance'] = $this->get_form_data($name.'_tolerance');
				$solver['relTol'] = $this->get_form_data($name.'_relTol');
				$solver['nPreSweeps'] = $this->get_form_data($name.'_nPreSweeps');
				$solver['nPostSweeps'] = $this->get_form_data($name.'_nPostSweeps');
				$solver['cacheAgglomeration'] = $this->get_form_data($name.'_cacheAgglomeration');
				$solver['agglomerator'] = $this->get_form_data($name.'_agglomerator');
				$solver['nCellsInCoarsestLevel'] = $this->get_form_data($name.'_nCellsInCoarsestLevel');
				$solver['mergeLevels'] = $this->get_form_data($name.'_mergeLevels');
				break;
			case 'smoothSolver':
				$solver['smoother'] = $this->get_form_data($name.'_smoother');
				$solver['tolerance'] = $this->get_form_data($name.'_tolerance');
				$solver['relTol'] = $this->get_form_data($name.'_relTol');
				$solver['nSweeps'] = $this->get_form_data($name.'_nSweeps');
				break;
		}

		return $solver;
	}

	private function get_form_solution($name, $info)
	{
		$solution = array();

		foreach ($this->CI->config->item('solution')[$info] as $key => $item)
		{
			$solution[$key] = $this->get_form_data($name.'_'.$key, $item['type']);
		}

		return $solution;
	}
}
