<?php

defined('BASEPATH') OR exit('No direct script access allowed');

class Solve_log
{
	protected $CI;

	public function __construct($params = NULL)
	{
		$this->CI =& get_instance();
	}

	public function get_chart_data($width = 200, $labels = 8)
	{
		static $color = array(
			'rgba(220,220,220,1)',	'rgba(151,187,205,1)',	'rgba(72,59,33,1)',		'rgba(69,81,204,1)',
			'rgba(211,43,134,1)',	'rgba(143,101,146,1)',	'rgba(55,240,125,1)',	'rgba(223,165,20,1)',
			'rgba(169,135,78,1)',	'rgba(179,232,251,1)',	'rgba(202,215,43,1)',	'rgba(233,215,69,1)'
		);

		$data = $this->load_chart_data($width, $labels);
		if (empty($data))
		{
			return NULL;
		}

		$chart = array(
			'labels'	=> $data['time'],
			'datasets'	=> array()
		);

		$idx = 0;
		foreach($data as $var => $vals)
		{
			if ($var != 'time')
			{
				$chart['datasets'][$idx] = array(
					'label'			=> $var,
					'strokeColor'	=> $color[$idx],
					'data'			=> $vals
				);
				$idx++;
			}
		}

		return json_encode($chart);
	}

	private function load_chart_data($width, $labels)
	{
		$logs = $this->CI->file_system->cat_data_file('monitor');
		if (empty($logs))
		{
			return NULL;
		}

		if (empty($logs[0]))
		{
			array_shift($logs);
		}

		$log_cnt = count($logs);
		if ($log_cnt < 1)
		{
			return NULL;
		}

		$cplot = $this->parse_plot($logs[0]);
		$ctime = floatval($cplot['time']);
		$minTime = $ctime;

		$lastLog = $this->parse_plot($logs[$log_cnt - 1]);
		if (!array_key_exists('time', $lastLog))
		{
			if ($log_cnt < 2)
			{
				return NULL;
			}

			$lastLog = $this->parse_plot($logs[$log_cnt - 2]);
		}

		$maxTime = $lastLog['time'];

		$step = ($maxTime - $minTime) / $labels;
		if ($step == 0)
		{
			$step = 1;
		}

		$unit = pow(10, floor(log10($step)));
		$mul = ceil($step / $unit);

		if ($mul > 8)
		{
			$step = 10;
		}
		elseif ($mul > 5)
		{
			$step = 8;
		}
		elseif ($mul > 4)
		{
			$step = 5;
		}
		elseif ($mul > 2)
		{
			$step = 4;
		}
		elseif ($mul > 1)
		{
			$step = 2;
		}
		else
		{
			$step = 1;
		}

		$step = $step * $unit;
		$pstep = $step * $labels / $width;
		$hstep = $pstep / 2.0;

		$time = ceil($minTime / $pstep) * $pstep;
		
		$data = array_fill_keys(array_keys($cplot), array());
		$cnt = 0;

		while ($line = next($logs))
		{
			$nplot = $this->parse_plot(trim($line));

			if (!array_key_exists('time', $nplot))
			{
				break;
			}
			$ntime = floatval($nplot['time']);
			$etime = $ntime - $ctime;

			if ($time > $maxTime)
			{
				break;
			}

			while ($time < $ntime)
			{
				foreach ($data as $var => $val)
				{
					$cval = floatval($cplot[$var]);
					if (array_key_exists($var, $nplot))
					{
						$nval = floatval($nplot[$var]);
					}
					else
					{
						$nval = $cval;
					}

					if ($var != 'time')
					{
						$cval = log10($cval + 1e-16);
						$nval = log10($nval + 1e-16);
					}

					$data[$var][$cnt] = $nval - ($nval - $cval) * ($ntime - $time ) / $etime;
				}

				$mod = fmod($data['time'][$cnt], $step);
				if ($mod >= $hstep && ($step - $mod) >= $hstep)
				{
					$data['time'][$cnt] = '';
				}

				$cnt++;
				$time += $pstep;
			}

			$cplot = $nplot;
			$ctime = $ntime;
		}

		if ($cnt < 1)
		{
			return NULL;
		}

		return $data;
	}

	private function parse_plot($line)
	{
		$data = array();

		$plots = explode(',', $line);

		foreach ($plots as $plot)
		{
			$p = explode('=', $plot);
			if (count($p) < 2)
			{
				return $data;
			}

			$data[$p[0]] = $p[1];
		}

		return $data;
	}
}
