<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Variable_input extends Foam_input
{
	protected $config;
	protected $dimensions;
	protected $boundaries;
	protected $inputs_info;

	public function __construct()
	{
		parent::__construct();
	}

	public function submit_form()
	{
		$data = array();

		$data['dimensions'] = $this->config['dimensions'];
		$data['internalField'] = 'uniform '.$this->get_form_field('internalField');
		$data['boundaryField'] = array();

		foreach($this->boundaries as $name => $info)
		{
			$data['boundaryField'][$name] = $this->get_form_boundary($name);
		}

		$this->CI->db->save_input_data($data);
	}

	public function get_js()
	{
		$js = "var input_class = '".$this->config['class']."'; var unit = '".$this->config['unit']."';";
		$js .= "$(document).ready(function() { rfields = jQuery.parseJSON('".json_encode($this->rfields)."'); $('form').submit(function(event) { if (!validate()) { event.preventDefault(); } });";
		$js .= "$(\"select[id$='_type']\").change(function() { onChangeType($(this)); }); });";
	
		return $js;
	}

	protected function init($params)
	{
		$this->config = $params['config'];
		$this->boundaries = $params['boundaries'];
		$this->inputs_info = $params['inputs_info'];

		$this->CI->load->library('variable_db', $params['config'], 'db');
	}

	protected function gen_inputs($values)
	{
		$this->CI->load->config('variable_conditions', TRUE);
		$this->CI->load->config('boundary_types', TRUE);

		$var_conds = $this->CI->config->item('variable_conditions')[$this->inputs_info['boundary_cond']]['conditions'];

		$this->make_input_internal_field($values);
		$boundaryFields = $this->extract_element('boundaryField', $values);
		foreach ($this->boundaries as $name => $info)
		{
			$type = element($this->config['object'], $var_conds[$info[$this->inputs_info['variables']]]);
			if (!$type)
			{
				$type = $this->CI->config->item('boundary_types')[$info['type']]['default'][$this->config['object']];
			}

			$this->make_boundary_inputs($name, $this->extract_element($name, $boundaryFields), $type);
		}
	}

	protected function make_input_types($name, $value)
	{
		$input = '<select name="'.$name.'" id="'.$name.'" class="form-control">';

		foreach ($this->config['types'] as $type)
		{
			$input .= '<option value="'.$type.'"'.($type == $value ? ' selected' : '').'>'.$type.'</option>';
		}
		$input .= '</select>';

		return $input;
	}
}
