var rfields = {};
var cfields = {};

function onChangeType(combo) {
    var boundary = $(combo)[0].id.slice(0, -5);
    var row = $(combo).closest('tr');
    $(row).siblings().remove();
    rfields[boundary] = {};
    
    switch (input_class)
    {
        case 'volVectorField':
            changeVectorInputs(boundary, combo[0].value, $(combo).closest('tbody'));
            break;
        case 'volScalarField':
            changeScalarInputs(boundary, combo[0].value, $(combo).closest('tbody'));
            break;
    }
}

function changeVectorInputs(boundary, type, tbody)
{
    switch (type)
    {
        case 'empty':
        case 'symmetry':
        case 'symmetryPlane':
        case 'zeroGradient':
            break;
        case 'fixedValue':
        case 'movingWallVelocity':
        case 'pressureInletOutletVelocity':
            $(tbody).append(vectorInput('Value (' + unit + ')', boundary, 'value'));
            break;
        case 'flowRateInletVelocity':
            $(tbody).append(scalarInput('Volumetric Flow Rate', boundary, 'volumetricFlowRate'));
            braek;
        case 'inletOutlet':
            $(tbody).append(vectorInput('Inlet Value (' + unit + ')', boundary, 'inletValue'));
            $(tbody).append(vectorInput('Value (' + unit + ')', boundary, 'value'));
            break;
        case 'surfaceNormalFixedValue':
            $(tbody).append(scalarInput('Ref Value (' + unit + ')', boundary, 'refValue'));
            break;
    }
}

function changeScalarInputs(boundary, type, tbody)
{
    switch (type)
    {
        case 'empty':
        case 'symmetry':
        case 'symmetryPlane':
        case 'zeroGradient':
        case 'kqRWallFunction':
        case 'omegaWallFunction':
            break;
        case 'calculated':
        case 'fixedValue':
            $(tbody).append(scalarInput('Value (' + unit + ')', boundary, 'value'));
            break;
        case 'inletOutlet':
            $(tbody).append(scalarInput('Inlet Value (' + unit + ')', boundary, 'inletValue'));
            break;
        case 'epsilonWallFunction':
        case 'epsilonStdWallFunction':
        case 'nutkWallFunction':
        case 'nutStdWallFunction':
            $(tbody).append(scalarInput('Cmu', boundary, 'Cmu', 0.09));
            $(tbody).append(scalarInput('kappa', boundary, 'kappa', 0.41));
            $(tbody).append(scalarInput('E', boundary, 'E', 9.8));
            break;
        case 'totalPressure':
            $(tbody).append(scalarInput('Gamma', boundary, 'gamma'));
            $(tbody).append(scalarInput('p0', boundary, 'p0'));
            break;
    }
}

function onChangeSolver(combo) {
    var variable = $(combo)[0].id.slice(0, -7);
    var row = $(combo).closest('tr');
    $(row).siblings(':gt(3)').remove();
    changeSolverInputs(variable, combo[0].value, $(combo).closest('tbody'));
}

function changeSolverInputs(variable, solver, tbody)
{
    var smoothers = ['GaussSeidel'];
    switch (solver)
    {
        case 'GAMG':
            $(tbody).append(selectInput('smoother', variable, 'smoother', smoothers));
            $(tbody).append(intInput('nPreSweeps', variable, 'nPreSweeps'));
            $(tbody).append(intInput('nPostSweeps', variable, 'nPostSweeps'));
            $(tbody).append(selectInput('cacheAgglomeration', variable, 'cacheAgglomeration', new Array('on', 'off')));
            $(tbody).append(selectInput('agglomerator', variable, 'agglomerator', new Array('dummy', 'faceAreaPair')));
            $(tbody).append(intInput('nCellsInCoarsestLevel', variable, 'nCellsInCoarsestLevel'));
            $(tbody).append(intInput('mergeLevels', variable, 'mergeLevels'));
            break;
        case 'smoothSolver':
		case 'NEXT::smoothSolver':
            $(tbody).append(selectInput('smoother', variable, 'smoother', smoothers));
            $(tbody).append(scalarInput('nSweeps', variable, 'nSweeps', '', true));
            break;
    }
}

function vectorInput(title, boundary, name, value, optional)
{
    name = boundary + '_' + name;
    if (!value && !optional) { value = [0, 0, 0]; }
    if (!optional) { rfields[boundary][name + '_x'] = title; rfields[boundary][name + '_y'] = title; rfields[boundary][name + '_z'] = title; }
    
    return '<tr><th>' + title + '</th><td><table style="width:100%"><tr><td><input type="number" name="' + name + '_x" id="' + name + '_x" class="form-control" value="' + value[0] + '" step="any" /></td><td><input type="number" name="' + name + '_y" id="' + name + '_y" class="form-control" value="' + value[1] + '" step="any" /></td><td><input type="number" name="' + name + '_z" id="' + name + '_z" class="form-control" value="' + value[2] + '" step="any" /></td></tr></table></td></tr>';
}

function scalarInput(title, boundary, name, value, optional)
{
    name = boundary + '_' + name;
    if (!optional) rfields[boundary][name] = title;
    if (value == undefined) value = '';

    return '<tr><th>' + title + '</th><td><input type="number" name="' + name + '" id="' + name + '" class="form-control" value="' + value + '" step="any" /></td></tr>';
}

function intInput(title, boundary, name, value, optional)
{
    name = boundary + '_' + name;
    if (!optional) rfields[boundary][name] = title;
    
    return '<tr><th>' + title + '</th><td><input type="number" name="' + name + '" id="' + name + '" class="form-control" value="' + value + '" min="0" step="1" /></td></tr>';
}

function selectInput(title, boundary, name, options)
{
    name = boundary + '_' + name;

    var html = '<tr><th>' + title + '</th><td><select name="' + name + '" id="' + name + '" class="form-control">';
    for (var i = 0; i < options.length; i++)
    {
        html += '<option value="' + options[i] + '">' + options[i] + '</option>';
    }
    html += '</select></td></tr>';
    
    return html;
}

function checksInput(title, boundary, name, values, cols)
{
    name = boundary + '_' + name;
    width = 100 / cols;
    cfields[boundary][name] = title;
    
    var html = '<tr><th style="verticla-align:top">' + title + '</th><td><table style="width:100%">';
    for (var i = 0; i < values.length; i++)
    {
        if (i % cols == 0) html += '<tr>';
        html += '<td style="width:' + width + '%"><input type="checkbox" name="' + name + '[]" id="' + name +'" value="' + values[i] + '"> ' + values[i] + '</td>';
        if (i % cols == cols - 1) html += '</tr>';
    }
    html += '</table></td></tr>';
    
    return html;
}

function validate()
{
    var ret = true;

    $.each(rfields, function(boundary, fields) {
        $.each(fields, function(field, name) {
            if (ret && $('#' + field).length && $('#' + field).val() == '') {
                if (boundary == '') alert('"' + name + '" field is required');
                else alert('"' + boundary + '.' + name + '" field is required');
                ret = false;
            }
        });
    });
    $.each(cfields, function(boundary, fields) {
        $.each(fields, function(field, name) {
            if (ret && $('#' + field).length && $('#' + field + ':checked').length == 0) {
                if (boundary == '') alert('"' + name + '" field is required');
                else alert('"' + boundary + '.' + name + '" field is required');
                ret = false;
            }
        });
    });

    return ret;
}

function removeItem(item)
{
    jQuery('#' + item).remove();
}

function changeContent(target, content)
{
    $(target).hide();
    $(content).show();
}

function initFields(r, c)
{
    $.each(r, function(boundary, fields) { rfields[boundary] = {}; $.each(fields, function(field, name) { rfields[boundary][field] = name; }); });
    $.each(c, function(boundary, fields) { cfields[boundary] = {}; $.each(fields, function(field, name) { cfields[boundary][field] = name; }); });
}
